@@ -19,7 +19,9 @@ Estimate_GERGM <- function(formula_object,
                            possible.stats,
                            GERGM_Object,
                            force_x_theta_updates,
-                           transformation_type) {
+                           transformation_type,
+                           verbose,
+                           force_x_lambda_updates) {
 
   # set the seed
   set.seed(seed)
@@ -32,7 +34,7 @@ Estimate_GERGM <- function(formula_object,
   net <- GERGM_Object@network
 
   rhs.formula <- possible.stats[statistics > 0]
-  rhs <- paste(rhs.formula, collapse = " + ")  #rewriting a formula for tnet
+  #rhs <- paste(rhs.formula, collapse = " + ")  #rewriting a formula for tnet
 
   # Flag if statistics do not meet requirement for Gibbs
   if (method == "Gibbs" & sum(GERGM_Object@weights != 1) > 0) {
@@ -71,21 +73,41 @@ Estimate_GERGM <- function(formula_object,
       if(MPLE.only == TRUE){
         cat("Updating Estimates -- Iteration:", i," \n")
         GERGM_Object <- store_console_output(GERGM_Object,paste("Updating Estimates -- Iteration:", i," \n"))
-        cat("Lambda Estimates", gpar$par,"\n")
+        if(verbose){
+          cat("Lambda Estimates", gpar$par,"\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("Lambda Estimates", gpar$par,"\n"))
-        gpar.new <- optim(par = as.numeric(gpar$par),
-                          llg,
-                          alpha = GERGM_Object@weights,
-                          theta = as.numeric(theta$par),
-                          z = GERGM_Object@data_transformation,
-                          method = "BFGS",
-                          together = together,
-                          GERGM_Object = GERGM_Object,
-                          hessian = T,
-                          control = list(fnscale = -1, trace = 6))
-        cat("Lambda estimates", "\n")
+        gpar.new <- NULL
+        if(verbose){
+          gpar.new <- optim(par = as.numeric(gpar$par),
+                            llg,
+                            alpha = GERGM_Object@weights,
+                            theta = as.numeric(theta$par),
+                            z = GERGM_Object@data_transformation,
+                            method = "BFGS",
+                            together = together,
+                            GERGM_Object = GERGM_Object,
+                            hessian = T,
+                            control = list(fnscale = -1, trace = 6))
+        }else{
+          gpar.new <- optim(par = as.numeric(gpar$par),
+                            llg,
+                            alpha = GERGM_Object@weights,
+                            theta = as.numeric(theta$par),
+                            z = GERGM_Object@data_transformation,
+                            method = "BFGS",
+                            together = together,
+                            GERGM_Object = GERGM_Object,
+                            hessian = T,
+                            control = list(fnscale = -1, trace = 0))
+        }
+        if(verbose){
+          cat("Lambda estimates", "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object, "Lambda estimates\n")
-        print(gpar.new$par)
+        if(verbose){
+          print(gpar.new$par)
+        }
         GERGM_Object <- store_console_output(GERGM_Object, toString(gpar.new$par))
         gpar.std.errors <- 1 / sqrt(abs(diag(gpar.new$hessian)))
         # Transform the unbounded weights to bounded weights via a t-distribution
@@ -96,10 +118,20 @@ Estimate_GERGM <- function(formula_object,
           BZ <- BZ + beta[j] * GERGM_Object@data_transformation[, , j]
         }
 
-        if(transformation_type == "logcauchy" | transformation_type == "cauchy"){
+        #store so we can transform back
+        GERGM_Object@BZ <- BZ
+        GERGM_Object@BZstdev <- sig
+
+        if(transformation_type == "logcauchy"){
+          GERGM_Object@bounded.network <- pst(log(net), BZ, sig, 1)
+        }
+        if( transformation_type == "cauchy"){
           GERGM_Object@bounded.network <- pst(net, BZ, sig, 1)
         }
-        if(transformation_type == "lognormal" | transformation_type == "gaussian"){
+        if(transformation_type == "lognormal"){
+          GERGM_Object@bounded.network <- pst(log(net), BZ, sig, Inf)
+        }
+        if( transformation_type == "gaussian"){
           GERGM_Object@bounded.network <- pst(net, BZ, sig, Inf)
         }
 
@@ -107,14 +139,30 @@ Estimate_GERGM <- function(formula_object,
         triples <- t(combn(1:num.nodes, 3))
         pairs <- t(combn(1:num.nodes, 2))
 
-        theta.new <- mple(GERGM_Object@bounded.network,
-                          statistics = GERGM_Object@stats_to_use,
-                          directed = directed)
-        cat("theta.new", theta.new$par, "\n")
+        if(GERGM_Object@is_correlation_network){
+          theta.new <- mple.corr(GERGM_Object@network,
+                                  GERGM_Object@bounded.network,
+                                  statistics = GERGM_Object@stats_to_use,
+                                  directed = directed,
+                                  verbose = verbose)
+        }else{
+          theta.new <- mple(GERGM_Object@bounded.network,
+                             statistics = GERGM_Object@stats_to_use,
+                             directed = directed,
+                             verbose = verbose)
+        }
+
+        if(verbose){
+          cat("theta.new", theta.new$par, "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("theta.new", theta.new$par, "\n"))
-        cat("theta", theta$par, "\n")
+        if(verbose){
+          cat("theta", theta$par, "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("theta", theta$par, "\n"))
-        cat("statistics", statistics, "\n")
+        if(verbose){
+          cat("statistics", statistics, "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("statistics", statistics, "\n"))
         theta.std.errors <- 1 / sqrt(abs(diag(theta.new$hessian)))
         GERGM_Object@theta.par <- theta.new$par
@@ -138,22 +186,34 @@ Estimate_GERGM <- function(formula_object,
             #if we reject any of the tests then convergence has not been reached!
             if(p.value2[i] < tolerance){count2[i] = 1}
           }
-          cat("Theta p.values", "\n")
+          if(verbose){
+            cat("Theta p-values", "\n")
+          }
           GERGM_Object <- store_console_output(GERGM_Object,paste("Theta p.values", "\n"))
-          print(p.value1)
+          if(verbose){
+            print(p.value1)
+          }
           GERGM_Object <- store_console_output(GERGM_Object,paste0(p.value1,collapse = " "))
-          cat("Lambda p.values", "\n")
+          if(verbose){
+            cat("Lambda p-values", "\n")
+          }
           GERGM_Object <- store_console_output(GERGM_Object,paste("Lambda p.values", "\n"))
-          print(p.value2)
+          if(verbose){
+            print(p.value2)
+          }
           GERGM_Object <- store_console_output(GERGM_Object,paste0(p.value2,collapse = " "))
           if (sum(count1) + sum(count2) == 0){
-            message("Parameter estimates have converged")
-            GERGM_Object <- store_console_output(GERGM_Object,"Parameter estimates have converged")
-            GERGM_Object@theta_estimation_converged <- TRUE
-            GERGM_Object@lambda_estimation_converged <- TRUE
-            theta <- theta.new
-            gpar <- gpar.new
-            break
+            message("Theta parameter estimates have converged...")
+            if(force_x_lambda_updates > i){
+              cat("Forcing",force_x_lambda_updates,"lambda updates...\n")
+            }else{
+              GERGM_Object <- store_console_output(GERGM_Object,"Parameter estimates have converged")
+              GERGM_Object@theta_estimation_converged <- TRUE
+              GERGM_Object@lambda_estimation_converged <- TRUE
+              theta <- theta.new
+              gpar <- gpar.new
+              break
+            }
           }
         }
         theta <- theta.new
@@ -162,22 +222,43 @@ Estimate_GERGM <- function(formula_object,
 
       if(MPLE.only != TRUE){
         # Estimate lambda
+
         cat("Updating Estimates -- Iteration:", i," \n")
+
         GERGM_Object <- store_console_output(GERGM_Object,paste("Updating Estimates -- Iteration:", i," \n"))
-        cat("Lambda Estimates", gpar$par,"\n")
+        if(verbose){
+          cat("Lambda Estimates", gpar$par,"\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("Lambda Estimates", gpar$par,"\n"))
-        gpar.new <- optim(par = as.numeric(gpar$par),
-                          llg,
-                          alpha = GERGM_Object@weights,
-                          theta = as.numeric(theta$par),
-                          z = GERGM_Object@data_transformation,
-                          method = "BFGS",
-                          together = together,
-                          GERGM_Object = GERGM_Object,
-                          hessian = T, control = list(fnscale = -1, trace = 6))
-        cat("Lambda estimates", "\n")
+        gpar.new <- NULL
+        if(verbose){
+          gpar.new <- optim(par = as.numeric(gpar$par),
+                            llg,
+                            alpha = GERGM_Object@weights,
+                            theta = as.numeric(theta$par),
+                            z = GERGM_Object@data_transformation,
+                            method = "BFGS",
+                            together = together,
+                            GERGM_Object = GERGM_Object,
+                            hessian = T, control = list(fnscale = -1, trace = 6))
+        }else{
+          gpar.new <- optim(par = as.numeric(gpar$par),
+                            llg,
+                            alpha = GERGM_Object@weights,
+                            theta = as.numeric(theta$par),
+                            z = GERGM_Object@data_transformation,
+                            method = "BFGS",
+                            together = together,
+                            GERGM_Object = GERGM_Object,
+                            hessian = T, control = list(fnscale = -1, trace = 0))
+        }
+        if(verbose){
+          cat("Lambda estimates", "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,"Lambda estimates\n")
-        print(gpar.new$par)
+        if(verbose){
+          print(gpar.new$par)
+        }
         GERGM_Object <- store_console_output(GERGM_Object,toString(gpar.new$par))
         gpar.std.errors <- 1 / sqrt(abs(diag(gpar.new$hessian)))
         # Transform the unbounded weights to bounded weights via a t-distribution
@@ -188,6 +269,9 @@ Estimate_GERGM <- function(formula_object,
           BZ <- BZ + beta[j] * transform.data[, , j]
         }
 
+        GERGM_Object@BZ <- BZ
+        GERGM_Object@BZstdev <- sig
+
         if(transformation_type == "logcauchy" | transformation_type == "cauchy"){
           GERGM_Object@bounded.network <- pst(net, BZ, sig, 1)
         }
@@ -214,7 +298,8 @@ Estimate_GERGM <- function(formula_object,
                              gain.factor = gain.factor,
 							               possible.stats = possible.stats,
 							               GERGM_Object= GERGM_Object,
-							               force_x_theta_updates = force_x_theta_updates)
+							               force_x_theta_updates = force_x_theta_updates,
+							               verbose = verbose)
 
         theta.new <- ret_list[[1]]
         GERGM_Object <- ret_list[[2]]
@@ -245,23 +330,35 @@ Estimate_GERGM <- function(formula_object,
           #if we reject any of the tests then convergence has not been reached!
           if(p.value2[i] < tolerance){count2[i] = 1}
         }
-        cat("Theta p.values", "\n")
+        if(verbose){
+          cat("Theta p.values", "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("Theta p.values", "\n"))
-        print(p.value1)
+        if(verbose){
+          print(p.value1)
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste0(p.value1,collapse = " "))
-        cat("Lambda p.values", "\n")
+        if(verbose){
+          cat("Lambda p.values", "\n")
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste("Lambda p.values", "\n"))
-        print(p.value2)
+        if(verbose){
+          print(p.value2)
+        }
         GERGM_Object <- store_console_output(GERGM_Object,paste0(p.value2,collapse = " "))
 
         if (sum(count1) + sum(count2) == 0){
-          message("Parameter estimates have converged")
-          GERGM_Object <- store_console_output(GERGM_Object,"Parameter estimates have converged")
-          GERGM_Object@theta_estimation_converged <- TRUE
-          GERGM_Object@lambda_estimation_converged <- TRUE
-          theta <- theta.new
-          gpar <- gpar.new
-          break
+          message("Theta parameter estimates have converged...")
+          if(force_x_lambda_updates > i){
+            cat("Forcing",force_x_lambda_updates,"lambda updates...\n")
+          }else{
+            GERGM_Object <- store_console_output(GERGM_Object,"Parameter estimates have converged")
+            GERGM_Object@theta_estimation_converged <- TRUE
+            GERGM_Object@lambda_estimation_converged <- TRUE
+            theta <- theta.new
+            gpar <- gpar.new
+            break
+          }
         }
         theta <- theta.new
         gpar <- gpar.new
@@ -292,16 +389,20 @@ Estimate_GERGM <- function(formula_object,
       ###
       statistics <- GERGM_Object@stats_to_use
       alphas <- GERGM_Object@weights
-      cat("Estimating Theta via MPLE... \n")
+      if(verbose){
+        cat("Estimating Theta via MPLE... \n")
+      }
       GERGM_Object <- store_console_output(GERGM_Object, "Estimating Theta via MPLE... \n")
       if(GERGM_Object@is_correlation_network){
         theta.init <- mple.corr(GERGM_Object@network, GERGM_Object@bounded.network,
                                 statistics = GERGM_Object@stats_to_use,
-                                directed = directed)
+                                directed = directed,
+                                verbose = verbose)
       }else{
         theta.init <- mple(GERGM_Object@bounded.network,
                            statistics = GERGM_Object@stats_to_use,
-                           directed = directed)
+                           directed = directed,
+                           verbose = verbose)
       }
 
       #print(theta.init)
@@ -328,7 +429,8 @@ Estimate_GERGM <- function(formula_object,
                            gain.factor = gain.factor,
                            possible.stats = possible.stats,
                            GERGM_Object = GERGM_Object,
-                           force_x_theta_updates = force_x_theta_updates)
+                           force_x_theta_updates = force_x_theta_updates,
+                           verbose = verbose)
 
       theta.new <- ret_list[[1]]
       GERGM_Object <- ret_list[[2]]
@@ -340,7 +442,6 @@ Estimate_GERGM <- function(formula_object,
   }
   theta <- t(as.matrix(theta$par))
   theta <- rbind(theta, theta.std.errors)
-  print(theta)
   colnames(theta) <- rhs.formula
   rownames(theta) <- c("est", "se")
   theta <- as.data.frame(theta)
