@@ -12,10 +12,14 @@ Trace_Plot <- function(GERGM_Object){
   nr <- nrow(GERGM_Object@network)
   normalizer <- nr*(nr-1)
   stats <- cbind(stats/normalizer,indexes)
+
+  actual_density <- GERGM_Object@stats[2,6]/normalizer
+
   p <- ggplot2::ggplot(stats, ggplot2::aes(x = indexes, y = edges))
   p  <- p + ggplot2::geom_line(color = UMASS_BLUE) +
     ggplot2::xlab("Iteration") +
     ggplot2::ylab("Network Density") +
-    ggplot2::ggtitle("Trace Plot of Density for Simulated Networks")
+    ggplot2::ggtitle("Trace Plot of Density for Simulated Networks") +
+    ggplot2::geom_abline(intercept = actual_density, slope = 0)
   print(p)
 }
